# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2010 E.Samyn - info@open-shift.com
# Author: E.Samyn
#
# This file is part of the project_quotation module
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU Affero General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program; 
# If not, see <http://www.gnu.org/licenses/> or  write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

from osv import osv, fields

class project_features_set(osv.osv):
    _name = "project.features_set"
    _description = "Features Set"
    _columns = {
        'name': fields.char('Features Set Name', size=64, required=True),
        'state' : fields.selection([('draft','Draft'),
								('to_be_confirmed','To be confirmed'),
								('valid','Valid'),], 'State'),
        'features_set_description': fields.text('Software description'),
		'features_set_current_stable_version': fields.char('Current stable version', size=64),
    }
    _order = 'name asc'

project_features_set()

#class project_software_version(osv.osv):
 #   _name = "project.features_set.version"
  #  _description = "software version"
   # _columns = {
#        'name': fields.char('Version', size=64, required=True),
        # 'state' : fields.selection([('development','Development'),
		#						('stable','Stable'),
		#						('obsolete','Obsolete'),], 'State'),
        # 'version_description': fields.text('Version description'),
 #   }

  #  _order = 'name asc'

#project_software_version()

class project_features_set_module(osv.osv):
    _name = "project.features_set.module"
    _description = "software module"
    _columns = {
        'name': fields.char('Module', size=64, required=True),
		'features_set_id': fields.many2one('project.features_set', 'Software', select=True),
        'module_type': fields.char('Type', size=64,help="ie: base, addons, extra addons,..."),
		'module_description': fields.text('Description'),
		'module_in_version': fields.char('Features Set versions', size=64, help="version separated by space or comma"),
    }

    _order = 'name asc'

project_features_set_module()

class features_set_use_profile(osv.osv):
    _name = "features_set.use.profile"
    _description = "Use profile = standard business type of use" # (exp: e-commerce, IT company, ...)
    _columns = {
        'name': fields.char('Profile name', size=64, required=True),
        'profile_description': fields.text('Use profile description'),
    }
    _order = 'name asc'
    
features_set_use_profile()

class project_feature(osv.osv):
    _name = "project.feature"
    _description = "Project Feature"
    _columns = {
        'name': fields.char('Feature name', size=250, required=True),
        'state' : fields.selection([('draft','Draft'),
									('tobevalidated','To be validated'),
									('valid','Valid'),
									('obsolete','Obsolete'),], 'State'),
        'feature_description': fields.text('Feature description', translate=True),
        'dependencies_ids': fields.many2many('project.feature', 'features_dependencies_rel', 'feature_id', 'feature_dep_id', "Dependencies"),
        'task_ids': fields.many2many('project.task', 'feature_task_rel', 'feature_id', 'task_id', "Required Tasks"),
        'opt_task_ids': fields.many2many('project.task', 'feature_opt_task_rel', 'feature_id', 'task_id', "Optional Tasks"),
        'feature_implem_duration': fields.float('Days min', digits=(12,2), required=True, help="Standard implementation min duration in days"),
        'feature_implem_duration_max': fields.float('Days max', digits=(12,2), required=True, help="Standard implementation max duration in days"),
        'feature_implem_complexity' : fields.selection([('1','1'),('3','3'),('5','5'),('7','7'),('10','10'),], 'Complexity', help="1 = in product & few time required ; 3 = in product & time required ; 5 = specific dev & few time required ; 7 = specific dev & time required ; 10 = specific dev & high complexity"),
        'feature_implem_method': fields.text('Feature implementation'),
        'feature_gui': fields.text('Feature GUI'),
        'feature_use': fields.text('Feature use'),
		'feature_scenario': fields.text('Test/use case', help="Test scenario / use case / example"),
        'features_set_id': fields.many2one('project.features_set', 'Features Set', ondelete='cascade',required=True,),
		'feature_block' : fields.many2one('project.functional_block', 'Category', ondelete='cascade', help="Functional block to sort out feaures"),
        'sequence': fields.integer('Sequence'),
        'feature_keywords': fields.char('Feature keywords', size=250),
        'feature_product_module': fields.many2one('project.features_set.module', 'Product module',),		
        'feature_in_version': fields.char('Features Set versions', size=64), #one2many('project.features_set.version', 'version_id', 'Versions'),
        'feature_hr_resource' : fields.many2one('product.product', 'Resource'),
        'feature_risk_level' : fields.selection([('minor','Minor'),
								   ('low','Low'),
								   ('significant','Significant'),
								   ('serious','Serious'),
								   ('critical','Critical')],"Risk"),
        'feature_is_base': fields.boolean('Base', help="If the feature has to be implement for minimum configuration"),
    }

    _defaults = {
        'state': lambda *a: 'draft',
		'feature_description': lambda *a: 'TODO',
		'feature_implem_method': lambda *a: 'TODO',
		'feature_gui': lambda *a: 'TODO',
		'feature_use': lambda *a: 'TODO',
		'feature_scenario': lambda *a: 'TODO',
		'feature_in_version': lambda *a: '6',
    }

    _order = 'feature_block,name asc'

project_feature()

class project_task(osv.osv):
    _inherit = 'project.task'
    _columns = {
                 'name': fields.char('Task Summary', size=128, required=True, translate=True),
                 'module_id': fields.many2one('project.features_set.module', 'Product module',),
                }

project_task()


class project_project(osv.osv):
    _inherit = 'project.project'
    _columns = {
                 'features_ids': fields.many2many('project.feature', 'project_feature_rel', 'project_id', 'feature_id', "Features"),
                }

    def generate_tasks(self, cr, uid, ids, context=None):
        for project in self.browse(cr, uid, ids):
            tasks = ([])
            for feature in project.features_ids:
                for task in feature.task_ids:
                    if task.id not in tasks:
                        tasks.add(task.id)
                        new_task = self.pool.get('project.task').copy(cr, uid, task.id, {'project_id': project.id})
                for task in feature.opt_task_ids:
                    if task.id not in tasks:
                        tasks.add(task.id)
                        new_task = self.pool.get('project.task').copy(cr, uid, task.id, {'project_id': project.id})
#                    tasks.add(task.id)
            return True

project_project()
